/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
//Name: Init
//When&What Calls: This is called when the window is first created
//Purpose: To get the window ready for doing operations
void kmpBrowse::init() {
    directoryView->addColumn("Directory");
    directoryView->setType(1);
    fileView->addColumn("Files");
    fileView->addColumn("Time");
    fileView->setType(2);
    fileView->setSelectionMode(QListView::Extended);
    //fileView->setSorting(-1, TRUE);
    resultsView->addColumn("File:");
    resultsView->addColumn("Track");
    resultsView->addColumn("Artist");
    resultsView->addColumn("Album");
    resultsView->addColumn("Title");
    resultsView->addColumn("Time");
    resultsView->setType(2);
    resultsView->setSelectionMode(QListView::Extended);
    resultsView->setShowSortIndicator(TRUE);
    //resultsView->setSorting(1, TRUE);
    QAccel * accel = new QAccel( streamView );
    accel->connectItem(accel->insertItem (Key_Delete), this, SLOT(deleteKeyPressed()));
    streamView->addColumn("Location");
    streamView->setSelectionMode(QListView::Extended);
    streamView->setType(2);
    
    icons * MyIcons;
    MyIcons = new icons;
    browseI = MyIcons->browseI;
    delete MyIcons;
    //First we set the current path to the root
    currentPath = "";
    //Intialize our connection to NULL
    conn = NULL;
    //We don't need to skip the click now (We skip the click event when you open a directory)
    skipClick=FALSE;
    //There is nothing selected last in the directory view
    lastSelected = NULL;
    //We aren't checking the status right now
    statusChecking = FALSE;
    //We get our anti-timeout timer up and running
    killTimeout = new QTimer(this);
    connect(killTimeout, SIGNAL(timeout()), this, SLOT(StatusCheck()));    
    unpauseStatus();
}

//Name: refreshView
//When&What Calls:  This is called when we first scan the root directory, and when we click refresh
//Purpose:  To get the contents of the root directory, and get our tree initialized
void kmpBrowse::refreshView() {
    //If we don't have a conenction, we just need to exit
    if(conn==NULL) return;
    //And we need to stop our anti-timeout
    pauseStatus();
    //There is again nothing selected
    lastSelected = NULL;
    //And we need to clear our 2 trees
    directoryView->clear();
    fileView->clear();
    //Some temporary variables
    dragListViewItem * ItemToAdd;
    mpd_InfoEntity * entity;
    //Add the root
    root = new QListViewItem(directoryView, "/");
    root->setDragEnabled(TRUE);
    root->setExpandable(TRUE);
    //And get the directorys and files in the root
    mpd_sendLsInfoCommand(conn, "");
    while((entity = mpd_getNextInfoEntity(conn))) {
 if(entity->type != MPD_INFO_ENTITY_TYPE_SONG && entity->type != MPD_INFO_ENTITY_TYPE_DIRECTORY ) {
     //This is a just in case measure, We shoule never get here
     mpd_freeInfoEntity(entity);
     continue;
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
     //If its a directory we add this directory to the root, with the folder icon
            mpd_SongStore directory;
            directory.file = QString(entity->info.directory->path);     
            ItemToAdd = new dragListViewItem(root, directory, nameOf(QString(entity->info.directory->path)));
     ItemToAdd->setPixmap(0,browseI);
     //And we are allowing them to be openable and DRAGABLE
     ItemToAdd->setDragEnabled(TRUE);
     ItemToAdd->setExpandable(TRUE);
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
     //If its a song we add it to the song side, and allow it do be dragged
            mpd_SongStore aSong;
            aSong.file = QString::fromUtf8(entity->info.song->file);
            aSong.artist = QString::fromUtf8(entity->info.song->artist);
            aSong.album = QString::fromUtf8(entity->info.song->album);
            aSong.title = QString::fromUtf8(entity->info.song->title);
            aSong.track = QString::fromUtf8(entity->info.song->track);
            aSong.time = entity->info.song->time;  
            ItemToAdd = new dragListViewItem(fileView, aSong, QString(nameOf(QString(entity->info.song->file))));
     ItemToAdd->setDragEnabled(TRUE);
 }
 //And we release the memory for this information
 mpd_freeInfoEntity(entity);
 
    }   

    //Next we open the root
    root->setOpen(TRUE);
    //And unpause the anti-timeout
    unpauseStatus();
}


//Name:  scanDirectory
//When&What Calls:  this is called when we open a directory
//Purpose: to scan a directory for files
void kmpBrowse::scanDirectory(QListViewItem * theItem) {
    mpd_SongStore aSong;
    //First we pause our status updates
    pauseStatus();
    //Clear the file view contents
    fileView->clear();
    //And get some variables ready
    dragListViewItem * ItemToAdd;
    mpd_InfoEntity * entity;
    //We are starting a variable called hasChildren... its intialized to false, and if 
    // this directory has subdirectorys then we set this to true, and if by the  end
    // of this we don't find subdirectorys then we make this directory non-expandable
    bool hasChildren = FALSE;
    //Then we check to see if we have already scanned this directory for sub-directorys
    bool cached = (bool)theItem->firstChild();
    //And we get the information of this directory
    mpd_sendLsInfoCommand(conn, (buildDirectoryPath(theItem)));
        checkForErrors();
    while((entity = mpd_getNextInfoEntity(conn))) {
 if(entity->type != MPD_INFO_ENTITY_TYPE_SONG && entity->type != MPD_INFO_ENTITY_TYPE_DIRECTORY ) {
     //This is a just in case measure, we should never get here
     mpd_freeInfoEntity(entity);
     continue;
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
     //We check here to see if this directory has already scanned the sub-directorys, and if it has
     // we just skip adding them to it
     if(cached) {
  //We skip adding directorys and release the memory, and move on
  mpd_freeInfoEntity(entity);
  //It obvoisly already has children
  hasChildren = TRUE;
  continue;
     }
     //Otherwise we add the subdirectorys as children to this folder
            mpd_SongStore directory;
            directory.file = QString(entity->info.directory->path);    
            ItemToAdd = new dragListViewItem(theItem, directory, nameOf(QString(entity->info.directory->path)));
     //Give them the folder icon
     ItemToAdd->setPixmap(0,browseI);
     //And Let them be dragable and expandable
     ItemToAdd->setDragEnabled(TRUE);
     ItemToAdd->setExpandable(TRUE);
     //And this directory has sub-directorys
     hasChildren = TRUE;
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
     //If its a song we add the song to the fileview side
     aSong.file = QString::fromUtf8(entity->info.song->file);
     aSong.artist = QString::fromUtf8(entity->info.song->artist);
     aSong.album = QString::fromUtf8(entity->info.song->album);
     aSong.title = QString::fromUtf8(entity->info.song->title);
     aSong.track = QString::fromUtf8(entity->info.song->track);
     aSong.time = entity->info.song->time;  
            ItemToAdd = new dragListViewItem(fileView, aSong, QString(nameOf(QString(entity->info.song->file))));
     //And let it be dragable
     ItemToAdd->setDragEnabled(TRUE);
 }
 //We release the memory here
 mpd_freeInfoEntity(entity);
    }
    //If by the end of this, we don't have any children, then we don't let this be expandalbe
    if(!hasChildren) theItem->setExpandable(FALSE);
    //And then we check to see if something was selected... 
    if(lastSelected) {
 //If something was selected we unselect it and repaint it
 lastSelected->setSelected(FALSE);
 directoryView->repaintItem(lastSelected);
    }
    //And now this is the last item selected
    lastSelected = theItem;
    //And we put in the path label the path of where the files are at
    pathLabel->setText("Path: /" + buildDirectoryPath(theItem));
    fileView->setPath(buildDirectoryPath(theItem));
    //And we finally unpuase the status
        checkForErrors();
    unpauseStatus();
}

//Name: collapse
//When&What Calls: This is called when the user collapses a directory
//Purpose: to unselect the last thing selected and view this directory
void kmpBrowse::collapse(QListViewItem * theItem) {
    skipClick = FALSE;
    //We view the contents of this directory
    viewDirectory(theItem);
    //And unselect the last item
    if(lastSelected) {
 lastSelected->setSelected(FALSE);
 directoryView->repaintItem(lastSelected);
    }
    // Now this is the last item selected
    lastSelected = theItem;
}

//Name: viewDirectory
//When&What Calls: this is called when a user clicks on an item, or collapses a directory
//Purpose: to view the contents of this directory in the fileview
void kmpBrowse::viewDirectory(QListViewItem * theItem) {
        checkForErrors();
    mpd_SongStore aSong;
    //If we didn't click anything, just skip this
    if(theItem == NULL) return;
    //If we need to skip this click we do so
    if(skipClick) {
 //And remeber that we skipped it
 skipClick = FALSE;
 return;
    }
    //Now we pasue the status updates
    pauseStatus();
    //Clear the file view
    fileView->clear();
    //Get some variables ready
    dragListViewItem * ItemToAdd;
    mpd_InfoEntity * entity;  
    //Get the information for this directory
    mpd_sendLsInfoCommand(conn, (buildDirectoryPath(theItem)));
    checkForErrors();
    //And we loop threw what we get
    while((entity = mpd_getNextInfoEntity(conn))) {
 if(entity->type != MPD_INFO_ENTITY_TYPE_SONG) {
     //We skip the directory items
     mpd_freeInfoEntity(entity);
     continue;
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
     //And add the songs to the file view
     aSong.file = QString::fromUtf8(entity->info.song->file);
     aSong.artist = QString::fromUtf8(entity->info.song->artist);
     aSong.album = QString::fromUtf8(entity->info.song->album);
     aSong.title = QString::fromUtf8(entity->info.song->title);
     aSong.track = QString::fromUtf8(entity->info.song->track);
     aSong.time = entity->info.song->time;
     ItemToAdd = new dragListViewItem(fileView, aSong, QString(nameOf(QString(entity->info.song->file))));
     //And allow them to be dragged
     ItemToAdd->setDragEnabled(TRUE);
 }
 //And release the memory
 mpd_freeInfoEntity(entity);
    }
    //Then we unselect the last item
    if(lastSelected) {
 lastSelected->setSelected(FALSE);
 directoryView->repaintItem(lastSelected);
    }
    //And make this the last selected item
    lastSelected = theItem;
    //and re-do the path label
    pathLabel->setText("Path: /" + buildDirectoryPath(theItem));
    fileView->setPath(buildDirectoryPath(theItem));
    //now we can unpause the status checks
        checkForErrors();
    unpauseStatus();
}

//Name: serverInfo
//When&What Calls: this is called to give us the info for the current server
//Purpose:
void kmpBrowse::serverInfo( QString serverName, int serverPort , QString serverPassword)
{
    pauseStatus();
    //If we already have a connection, close it
    if(!(conn==NULL)) {
 mpd_closeConnection(conn);
 conn = NULL;
    }
    //And open the new server
    conn = mpd_newConnection (serverName, serverPort, 5);
    if(serverPassword != "") {
 mpd_sendPasswordCommand(conn, serverPassword);
 checkForErrors();
 mpd_finishCommand(conn);
 checkForErrors();
    }
    //And make sure we connected good
    checkForErrors();
    //And then refresh the root directory
    refreshView();
    unpauseStatus();
    
}






//Name: checkForErrors
//When&What Calls: this is called several times
//Purpose:  to check for errors in the pipe
void kmpBrowse::checkForErrors()
 {
    //If we have an error
    if(conn && conn->error) {
        
 //Get the error
 QString Error;
 Error = QString(conn->errorStr);
 printf("ERROR:%s!\n", Error.ascii());
 if(conn->error == MPD_ERROR_ACK) {
     //If its an acknodedgement we ignore
     //&&&&&&&&&&&&&&&&&DEBUG&&&&&&&&&&&&&&&&&&&&
     pathLabel->setText("Error: " + Error.right(Error.length()-4));
        //&&&&&&&&&&&&&&&&&DEBUG&&&&&&&&&&&&&&&&&&&&
     mpd_clearError(conn);
     return;
 } 
 //If we do have an error we close the pipe, and tell the main window to reconnect
 mpd_clearError(conn);
 mpd_closeConnection(conn);
 conn =NULL;
 killTimeout->stop();
 //This informs the main window that we have an error
 connError();
    }
}
//Name: nameOf
//When&What Calls: this is called to recursively find the name 
//Purpose:  To get the name of the directory or song (everything right of the last /)
QString kmpBrowse::nameOf(QString thePath) {
    //first we find the last /
    int i = thePath.findRev( '/' );
    //and return everythin to the right of it
    return(thePath.right(thePath.length() - ++i));
}





//Name: StatusCheck
//When&What Calls: this is called by the anti-timeout timer
//Purpose: this is called to stop the connection from timing out
void kmpBrowse::StatusCheck()
{
    //If there isn't a connection we just skip this
    if(!conn) return;
    //Otherwise we start checking
    statusChecking = TRUE;  
    //Get a the status
    mpd_sendStatusCommand(conn);
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();
    checkForErrors();
    //And continue running
    statusChecking = FALSE;
}

//Name: closeConnection
//When&What Calls: this is called by the main window when it loses its connection
//Purpose: To close the browse windows connection
void kmpBrowse::closeConnection()
{
    killTimeout->stop();
    if(conn) mpd_closeConnection(conn);
    conn =NULL;
}



//Name: enable Me
//When&What Calls: this is called by the main window
//Purpose: to set the state of all of the widgets
void kmpBrowse::enableMe(bool status)
{
    //We just set all of the widgets enabledness to that of status
    pathLabel->setEnabled(status);
    directoryView->setEnabled(status);
    fileView->setEnabled(status);
    combo_searchtype->setEnabled(status);
    le_search->setEnabled(status);
    btn_search->setEnabled(status);   
    resultsView->setEnabled(status);
}




//Name:  closeEvent
//When&What Calls: this is called when the browse window gets the close event 
//Purpose:  To tell the main window that we have closed
void kmpBrowse::closeEvent( QCloseEvent * a) {
    //This is connected to the main window and notifies it that we are closing
    browseClosed();
    //Accept the close event
    a->accept();
}

//Name: pauseStatus
//When&What Calls: this is called several times
//Purpose: to stop the status from checking when we do anything with our pipe
void kmpBrowse::pauseStatus() {
    //If we are already checking the status, we wait until it finishes
    while(statusChecking) {}
    //Then we stop it
    killTimeout->stop();
}

//Name: unpauseStatus
//When&What Calls: this is called several times
//Purpose: to start up the timeout timer
void kmpBrowse::unpauseStatus() {
    killTimeout->start(5000);
}


void kmpBrowse::search()
{
        checkForErrors();
    pauseStatus();
    mpd_SongStore aSong;
    btn_search->setCaption(QString("Searching..."));
    btn_search->setEnabled(0);
    //Clear the file view
    resultsView->clear();
    bool itemFound = FALSE;
    //Get some variables ready
    dragListViewItem * ItemToAdd;
    mpd_InfoEntity * entity;  
    //Get the information for this directory
    mpd_sendSearchCommand(conn, combo_searchtype->currentItem(), (le_search->text()));
    checkForErrors();
    while((entity = mpd_getNextInfoEntity(conn))) {
 if(entity->type != MPD_INFO_ENTITY_TYPE_SONG) {
     //We skip the directory items
     mpd_freeInfoEntity(entity);
     continue;
 }
 if(entity->type == MPD_INFO_ENTITY_TYPE_SONG) {
     itemFound = TRUE;
     //And add the songs to the file view
     aSong.file = QString::fromUtf8(entity->info.song->file);
     aSong.artist = QString::fromUtf8(entity->info.song->artist);
     aSong.album = QString::fromUtf8(entity->info.song->album);
     aSong.title = QString::fromUtf8(entity->info.song->title);
     aSong.track = QString::fromUtf8(entity->info.song->track);
     aSong.time = entity->info.song->time;
     ItemToAdd = new dragListViewItem(resultsView, resultsView->lastItem(), aSong, fileTitle(aSong.file), aSong.track, aSong.artist, aSong.album, aSong.title, minTime(aSong.time));
     //And allow them to be dragged
     ItemToAdd->setDragEnabled(TRUE);
 }
 //And release the memory
 mpd_freeInfoEntity(entity);
    }
    
    btn_search->setCaption(QString("Search"));  
    btn_search->setEnabled(1);  
    if(!itemFound) {
        QListViewItem * noFound = new QListViewItem(resultsView, "NO Songs Found", "","", "", "", "");
        noFound->setEnabled(FALSE);
    }
        checkForErrors();
    unpauseStatus();
}





void kmpBrowse::addStream()
{
    dragListViewItem * ItemToAdd;
    if( le_stream->text() != "") {
        mpd_SongStore aSong;
        aSong.file = le_stream->text();
        aSong.artist = "";
        aSong.album = "";
        aSong.title = "";
        aSong.track = "";
        aSong.time = 0;
        ItemToAdd = new dragListViewItem(streamView, streamView->lastItem(), aSong, le_stream->text());
        ItemToAdd->setDragEnabled(TRUE);
    }
}


QStringList kmpBrowse::getStreams()
{
    QStringList names;
    dragListViewItem * curItem = (dragListViewItem * ) streamView->firstChild();
    while(curItem)
    {
        names += curItem->getItem().file;
        curItem = (dragListViewItem *) curItem->itemBelow();
    }
    return names;
}


void kmpBrowse::setStreams( QStringList list )
{
    dragListViewItem * ItemToAdd;
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
        mpd_SongStore aSong;
        aSong.file = *it;
        aSong.artist = "";
        aSong.album = "";
        aSong.title = "";
        aSong.track = "";
        aSong.time = 0;
        ItemToAdd = new dragListViewItem(streamView, streamView->lastItem(), aSong, *it);
        ItemToAdd->setDragEnabled(TRUE);
    }

}


void kmpBrowse::deleteKeyPressed()
{
    dragListViewItem * nextItem;
    dragListViewItem * curItem = (dragListViewItem * ) streamView->firstChild();
    while(curItem)
    {
        if(curItem->isSelected()) {
            nextItem = (dragListViewItem *) curItem->itemBelow();
            streamView->takeItem(curItem);
        } else nextItem = (dragListViewItem *) curItem->itemBelow();
        curItem = nextItem;
    }
}
